--[[
	(C) 2014-2016 J.J. Snelgrove <jj@stormlord.ca>

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

local FOLDER, guids = ...

local squelch

local function FullName(name)
	if type(name) ~= "string" or name == "" then
		return nil
	elseif name:find("-", nil, true) then
		return name
	elseif realm and realm ~= "" then
		return FULL_PLAYER_NAME:format(name, (realm:gsub("%s*%-*", "")))
	end
	return FULL_PLAYER_NAME:format(name, (GetRealmName():gsub("%s*%-*", "")))
end

local function MessageEventFilter_Squelch(self, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, ...)
	local success, class, classFile, race, raceFile, sex, name, realm = pcall(GetPlayerInfoByGUID, arg12)
	if not success then
		return false
	end
	local sender = FullName(name, realm)
	guids[sender] = arg12
	return squelch[sender] or false
end

local toRemove
local function RemoveByAccessID(text, r, g, b, chatTypeID, lineID, accessID)
	return toRemove == accessID
end

local function Squelch(sender)
	squelch[sender] = true
	accessIDs = ChatHistory_GetAllAccessIDsByChanSender(guids[sender])
	for i = 1, NUM_CHAT_WINDOWS do
		local frame = _G[("ChatFrame%d"):format(i)]
		if frame then
			for i, accessID in ipairs(accessIDs) do
				toRemove = accessID
				frame:RemoveMessagesByPredicate(RemoveByAccessID)
				toRemove = nil
			end
		end
	end
	SendSystemMessage(("%s is now being squelched."):format(Ambiguate(sender, "none")))
end

StaticPopupDialogs["SQUELCH_CONFIRM"] = {
	text = "Do you want to squelch \"%s\"?",
	button1 = YES,
	button2 = NO,
	showAlert = true,
	enterClicksFirstButton = true,
	whileDead = true,
	hideOnEscape = true,
	OnAccept = function(self, sender)
		Squelch(sender)
	end,
}
		
local function UnitPopup_OnClick_Hook(self)
	if self.value == "SQUELCH" then
		local sender = FullName(UIDROPDOWNMENU_INIT_MENU.name)
		StaticPopup_Show("SQUELCH_CONFIRM", Ambiguate(sender, "none"), nil, sender)
	end
end

local playerName = FullName(UnitName("player"))
local function UnitPopup_HideButtons_Hook()
	if UIDROPDOWNMENU_INIT_MENU.which ~= "FRIEND" or UIDROPDOWNMENU_MENU_VALUE and UIDROPDOWNMENU_MENU_VALUE ~= "FRIEND" then
		return
	end
	for i, button in ipairs(UnitPopupMenus["FRIEND"]) do
		if button == "SQUELCH" then
			local sender = FullName(UIDROPDOWNMENU_INIT_MENU.name)
			if not sender or not guids[sender] or sender == playerName or squelch[sender] then
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][i] = 0
			end
			break
		end
	end
end

SLASH_SQUELCH1 = "/squelch"
SlashCmdList["SQUELCH"] = function(input, editBox)
	local command = input:match("^([^%s]+)")
	if not command or command == "" then
		SendSystemMessage("Squelch usage:")
		SendSystemMessage("- /squelch <Name> - add/remove a squelch")
		SendSystemMessage("- /squelch <Name-RealmName> - add/remove a crossrealm squelch")
		SendSystemMessage("- /squelch clear - clear all squelches")
	elseif command:lower() == "clear" then
		table.wipe(squelch)
		SendSystemMessage("All squelches have been cleared.")
	else
		local name = FullName(command:gsub("^%l", string.upper))
		if not squelch[name] then
			if name == playerName then
				SendSystemMessage("You can't squelch yourself.")
			else
				Squelch(name)
			end
		else
			squelch[name] = nil
			SendSystemMessage(("%s is no longer being squelched."):format(Ambiguate(name, "none")))
		end
	end
end

UnitPopupButtons["SQUELCH"] = { text = "Squelch", dist = 0 }
local insertPoint = 1
for i, button in ipairs(UnitPopupMenus["FRIEND"]) do
	if button == "TARGET" or button == "PVP_REPORT_AFK" then
		table.remove(UnitPopupMenus["FRIEND"], i)
		if insertPoint then
			insertPoint = insertPoint - 1
		end
	elseif button == "IGNORE" then
		table.insert(UnitPopupMenus["FRIEND"], insertPoint + i, "SQUELCH")
		insertPoint = nil
	end
end

hooksecurefunc("UnitPopup_OnClick", UnitPopup_OnClick_Hook)
hooksecurefunc("UnitPopup_HideButtons", UnitPopup_HideButtons_Hook)

--ChatFrame_AddMessageEventFilter("CHAT_MSG_ACHIEVEMENT", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", MessageEventFilter_Squelch)
--ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL_JOIN", MessageEventFilter_Squelch)
--ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL_LEAVE", MessageEventFilter_Squelch)
--ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL_NOTICE_USER", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_EMOTE", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD", MessageEventFilter_Squelch)
--ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD_ACHIEVEMENT", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT_LEADER", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_OFFICER", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY_LEADER", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_TEXT_EMOTE", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", MessageEventFilter_Squelch)
ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL", MessageEventFilter_Squelch)

local Loader = CreateFrame("Frame")
Loader:SetScript("OnEvent", function(self, event, addon, ...)
	if event == "ADDON_LOADED" and addon == FOLDER then
		if not squelchSaved or not squelchSaved.lastLogout or squelchSaved.lastLogout + 900 < time() then
			squelchSaved = {}
		end
		squelch = squelchSaved
	elseif event == "PLAYER_LOGOUT" then
		squelch.lastLogout = time()
	end
end)
Loader:RegisterEvent("ADDON_LOADED")
Loader:RegisterEvent("PLAYER_LOGOUT")
